// Interacting with docker containers
import * as dockerLabels from "./labels.js";
import Dockerode from "dockerode";

export class DockerClient {
    #dockerConnection;

    /**
     * 
     * @param {Dockerode} dockerConnection 
     */
    constructor(dockerConnection) {
        this.#dockerConnection = dockerConnection;
    }

    /**
     * Start a new deployment if it not already exist.
     * @param {string} userId 
     * @param {string} productId 
     * @param {string} orderId 
     */
    async start(userId, productId, orderId) {
        if (await this.#existContainer(userId, productId, orderId)) return;
        await this.#dockerConnection.pull("hashicorp/http-echo");
        const container = await this.#dockerConnection.createContainer({
            Image: "hashicorp/http-echo:latest",
            Cmd: "-text='hellow'",
            Labels: {
                [dockerLabels.appContainerLabelKeyProject]: "",
                [dockerLabels.appContainerLabelKeyUserId]: userId,
                [dockerLabels.appContainerLabelKeyProductId]: productId,
                [dockerLabels.appContainerLabelKeyOrderId]: orderId
            }
        });
        await container.start();
    }

    /**
     * Delete an existing deployment.
     * @param {string} orderId
     */
    async delete(orderId) {
        const containers = await this.#dockerConnection.listContainers({
            filters: {
                label: [
                    dockerLabels.appContainerLabelKeyProject,
                    `${dockerLabels.appContainerLabelKeyOrderId}=${orderId}`
                ]
            }
        });
        if (containers.length != 1) {
            if (containers.length < 1) {
                throw new Error("No container for the order " + orderId + " found");
            }
            throw new Error("Too many container for the order " + orderId + " found");
        }
        const containerInfo = containers[0];
        const containerId = containerInfo.Id;
        const container = await this.#dockerConnection.getContainer(containerId);

        await container.stop({t: 10});
        await container.remove();
    }

    /**
     * Check if the deployment of the order is healthy.
     * @param {string} orderId
     * @returns health status
     */
    async isHealthy(orderId) {
        const foundContainers = await this.#searchContainerByOrderId(orderId);
        // we currently only expect one container
        if (foundContainers.length != 1) {
            return false;
        }

        return true;
    }

    async #existContainer(userId, productId, orderId) {
        const foundContainers = await this.#dockerConnection.listContainers({
            filters: {
                label: [
                    dockerLabels.appContainerLabelKeyProject,
                    `${dockerLabels.appContainerLabelKeyUserId}=${userId}`,
                    `${dockerLabels.appContainerLabelKeyProductId}=${productId}`,
                    `${dockerLabels.appContainerLabelKeyOrderId}=${orderId}`
                ],
                status: [
                    "created",
                    "restarting",
                    "running",
                    "removing",
                    "paused",
                    "exited",
                    "dead"
                ]
            }
        });
        if (foundContainers.length == 0) return false;
        return true;
    }

    async #searchContainerByOrderId(orderId) {
        return await this.#dockerConnection.listContainers({
            filters: {
                label: [
                    dockerLabels.appContainerLabelKeyProject,
                    `${dockerLabels.appContainerLabelKeyOrderId}=${orderId}`
                ],
                status: [
                    "created",
                    "restarting",
                    "running",
                    "removing",
                    "paused",
                    "exited",
                    "dead"
                ]
            }
        });
    }
}
