import express from "express";
import { Docker } from "./docker-helper.cjs";
import { DockerClient } from "./docker.js";
import * as fs from "node:fs"

const { dockerClient, app } = init();
const port = 3000;

app.use(express.text({ type: '*/*' }))
app.use(express.json())

app.get('/', async (req, res) => {
    console.log(req.query)
    console.log(await docker.version());
    res.send('Hello World!');
})

app.post('/', async (req, res) => {
    console.log(req.query)
    console.log(req.body)
    res.send('Hello World!');
})

app.route("/api/v1/order/:id")
    // create a new deployment
    .post(async (req, res) => {
        try {
            const queryParams = req.query;
            if (queryParams.userId == null || queryParams.productId == null) {
                res.status(400).send("Missing query params. Required: userId, productId")
                return
            }
            const params = req.params;
            await dockerClient.start(queryParams.userId, queryParams.productId, params.id);
        } catch (e) {
            res.status(500).send(e.message);
            return;
        }
        res.status(201).end();
        return;
    })
    // delete an order
    .delete(async (req, res) => {
        const params = req.params;
        const orderId = params.id;
        try {
            await dockerClient.delete(orderId);
        } catch (e) {
            if (e.message.includes("no such") || e.message.includes("No container")) {
                res.status(404).send("No order found with id: " + orderId);
                return;
            }
            res.status(500).send(e.message);
            return;
        }
        res.status(204).end();
        return;
    })

app.get("/api/v1/order/:id/health", async (req, res) => {
    const params = req.params;
    const orderId = params.id;
    try {
        const isHealthy = await dockerClient.isHealthy(orderId);
        if (!isHealthy) {
            res.status(404).send("Not healthy");
            return;
        }
    } catch (e) {
        res.status(500).send(e.message);
        return;
    }
    res.status(200).end();
});

app.listen(port, () => {
    console.log(`Example app listening on port ${port}`);
})


/**
 * Contains the initialization logic.
 * @returns {Express} express session
 */
function init() {
    var socket = process.env.DOCKER_SOCKET || '/var/run/docker.sock';
    var stats = fs.statSync(socket);

    if (!stats.isSocket()) {
        throw new Error('Are you sure the docker is running?');
    }
    const docker = Docker({ socketPath: '/var/run/docker.sock' });
    const dockerClient = new DockerClient(docker);
    const app = express();

    return { dockerClient: dockerClient, app: app };
}
